// pin
#define BUZ_PIN     D6 
#define GAS_PIN     ###
#define GAS_ON_SEC  10
#define GAS_TRIGGER 500
#define NAME_TRIGGER "trigger"

// topic 
#define PREFIX               "alarm2"
#define TOPIC_GAS            "gas"
#define TOPIC_REPORT_ALARM   "report-alarm"
#define TOPIC_REPORT_ALARM_R "report-alarm-r"

// Aim 
#include <AimOutput.h>
#include <AimAlarm.h>
#include <AimConfig.h>
#include <AimSerial.h>
#include <AimTimer.h>

#include <myConfig.h>
#include <AimMqtt.h>

AimOutput buz(BUZ_PIN,HIGH);
AimAlarm  gas(GAS_PIN,###,GAS_ON_SEC);
AimMqtt   mqtt;
AimConfig config("my-config.json");
AimSerial mySerial;
AimTimer  timerGas(0.5);
AimTimer  timerPrint(5);

void setup() {
  Serial.begin(115200);
  Serial.println();

  Serial.println("Input > 'trigger nnn' to set new trigger");

  getNewTrigger(####);

  mqtt.begin(myMqttSub,myMqttCallback);
  mySerial.begin(mySerialCallback);  
}

void loop() {
  mqtt.run();
  buz.run();
  mySerial.run();

  if (timerGas.isOn()) {
    if (gas.###()) {
      mqtt.pub(TOPIC_GAS,"ON");
      buz.set(300,200,###,-1,1000);
    }
    else if(gas.###()) { 
      mqtt.pub(TOPIC_GAS,"OFF");
      buz.off();
    }
  }

  if (timerPrint.isOn()) {
    Serial.printf("Gas pinRead(0~1024)=%d\n",gas.read());
  }
}

void myMqttSub() {
  mqtt.sub(TOPIC_REPORT_ALARM,1);
}

void myMqttCallback(const String& topic,const String& payload) {
  if (topic == TOPIC_REPORT_ALARM) {
    bool reportAlarm = mqtt.toBool(payload);
    mqtt.pub(TOPIC_REPORT_ALARM_R,reportAlarm); 

    if (reportAlarm) {
      gas.###();
    }
    else {
      gas.####();
    }     
  }
}

void mySerialCallback(const String& key,const String& val) {
  if (key == "alarm") {
    mqtt.pub(TOPIC_REPORT_ALARM,mqtt.toBool(val),true);
  }
  else if (key == "trigger") {
    setNewTrigger(val);
  }
}

void getNewTrigger(int trigger) {
  config.sync###(NAME_TRIGGER,trigger);
  gas.###Trigger(trigger);
  Serial.printf("gas trigger=%d\n",gas.###Trigger());
}

void setNewTrigger(const String& val) {
  int trigger = val.to###();
  config.setInt(NAME_TRIGGER,trigger);
  gas.###Trigger(trigger);
  Serial.printf("gas trigger=%d\n",gas.###Trigger());
}